package ru.ulstu.extractor.generation.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.generation.model.GenerationForm;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;
import ru.ulstu.extractor.ts.util.DateUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class GenerationService {
    private final TimeSeriesService timeSeriesService;
    private final BranchService branchService;

    public GenerationService(TimeSeriesService timeSeriesService,
                             BranchService branchService) {
        this.timeSeriesService = timeSeriesService;
        this.branchService = branchService;
    }

    public void generate(GenerationForm generationForm) {
        Branch branch = branchService.findByBranchId(Branch.GENERATED_BRANCH_ID).orElseThrow();
        Date startDate = DateUtils.addDays(-generationForm.getTsLength());
        Arrays.stream(TimeSeriesType.values()).forEach(tsType -> {
            List<TimeSeriesValue> tsValues = new ArrayList<>();
            for (int i = 0; i < generationForm.getTsLength(); i++) {
                tsValues.add(new TimeSeriesValue(DateUtils.addDays(startDate, i),
                        getNextAdditiveValue(tsValues, generationForm)));
            }
            final String tsName = "Генерированный " + tsType.getDescription();
            timeSeriesService.save(tsName, branch, tsType, tsValues);
        });
    }

    private double getNextAdditiveValue(List<TimeSeriesValue> timeSeriesValues, GenerationForm generationForm) {
        double newValue;
        int maxTryCount = 10;
        do {
            double oneStepDiff = generationForm.getRandomWeight() * (Math.random() - 0.5) * (generationForm.getMax() - generationForm.getMin()) / (generationForm.getTsLength() + 10);
            if (timeSeriesValues.isEmpty()) {
                if (generationForm.getBaseTendency() > 0) {
                    newValue = 0.1 * (generationForm.getRandomWeight() > 0.0 ? oneStepDiff : 1.0) * (generationForm.getMax() - generationForm.getMin());
                } else if (generationForm.getBaseTendency() < 0) {
                    newValue = generationForm.getMax() - 0.1 * (generationForm.getRandomWeight() > 0.0 ? oneStepDiff : 1.0) * (generationForm.getMax() - generationForm.getMin());
                } else {
                    newValue = generationForm.getBase();
                }
            } else {
                newValue = timeSeriesValues.get(timeSeriesValues.size() - 1).getValue()
                        + (generationForm.getRandomWeight() > 0.0 ? oneStepDiff : 1.0) * generationForm.getBaseTendency()
                        * ((generationForm.getMax() - generationForm.getMin()) / (generationForm.getTsLength() + 10))
                        * generationForm.getTendencyDynamic();
            }
            maxTryCount--;
        } while (((newValue <= generationForm.getMin())
                || (newValue >= generationForm.getMax()))
                && (maxTryCount > 0));
        return newValue;
    }
}
