package ru.ulstu.extractor.ts.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.ts.creator.scheduled.ScheduledTimeSeriesCreator;

import java.util.List;
import java.util.stream.Collectors;

import static ru.ulstu.extractor.branch.model.Branch.GENERATED_BRANCH_ID;

@Service
public class ScheduledTimeSeriesService {
    private final Logger log = LoggerFactory.getLogger(ScheduledTimeSeriesService.class);
    private final List<ScheduledTimeSeriesCreator> scheduledTimeSeriesCreators;
    private final BranchService branchService;

    public ScheduledTimeSeriesService(List<ScheduledTimeSeriesCreator> scheduledTimeSeriesCreators,
                                      BranchService branchService) {
        this.scheduledTimeSeriesCreators = scheduledTimeSeriesCreators;
        this.branchService = branchService;
    }

    @Scheduled(cron = "0 0 8 * * *")
    public void addTimeSeriesPoints() {
        log.info("Старт добавления новых точек временного ряда");
        List<Branch> branches = branchService.findAll()
                .stream()
                .filter(b -> !b.getId().equals(GENERATED_BRANCH_ID))
                .collect(Collectors.toList());
        branches.forEach(branch -> {
            scheduledTimeSeriesCreators.forEach(creator -> creator.addTimeSeriesValue(branch));
        });
        log.info("Завершение добавления новых точек временного ряда");
    }
}
